#!/bin/bash

PATH=/usr/local/bin:/usr/local/sbin:/usr/bin:/usr/sbin:/bin:/sbin

if [ -f $0 ] ; then
	chmod +x $0
fi

if [ ! -d /dev/net ] ; then
	mkdir -p /dev/net
	mknod /dev/net/tun c 10 200
fi

workdir=/etc/openvpn
ovpn=$(which openvpn)
args="--daemon --config"
runthis="$ovpn $args"
pkill=$(which pkill)
killall=$(which killall)

function pingit {
    useDevice=$1
	deadline=10
	count=5
	timeout=10
	pinghosts="192.36.125.18 8.8.8.8"
	successCount=0
	failCount=0
	#echo "Checking network ..."
	for host in $pinghosts
	do
		#echo -n "."
		totalCount=$(expr $totalCount + 1)
		response=$(ping -c${count} -w${deadline} -W${timeout} -I $useDevice $host)
		checkZero=$(echo $response | grep "0 received")
		if [ "" != "$checkZero" ] ; then
			failCount=$(expr $failCount + 1)
		else
			successCount=$(expr $successCount + 1)
		fi
	done
	#echo ""
	#echo "Successful pings: ${successCount} of ${totalCount}"
	if [ "$successCount" != "$totalCount" ] && [ "$successCount" = "0" ] ; then
		doRestart=1
		#echo "[OpenVPN Automation] Network Down. Restart required!"
		if [ "" != "$pkill" ] && [ -f $pkill ] ; then
			#echo "[OpenVPN Automation] ${pkill} used to kill openvpn ..."
			${pkill} -9 openvpn
		else
			if [ "${killall}" != "" ] ; then
				#echo "[OpenVPN Automation] ${killall} used to kill openvpn ..."
				${killall} -9 openvpn >/dev/null 2>&1
			fi
		fi
		#echo "[OpenVPN Automation] killswitch used to clear default route ..."
		ip route del default >/dev/null 2>&1 || true
		echo "0"
	else
		echo "1"
	fi
}

if [ "" = "$ovpn" ] ; then
	echo "[Missing OpenVPN] Making sure dpkg has no damaged packages..."
	dpkg --configure -a >/dev/null 2>&1
	echo "[Missing OpenVPN] Installing OpenVPN..."
	apt-get -y install openvpn >/dev/null 2>&1
	if [ "$?" != "0" ] ; then
		echo "[Missing OpenVPN] Install failure."
		exit
	fi
	ovpn=$(which openvpn)
	if [ "" = "$ovpn" ] ; then
		echo "[Missing OpenVPN] Still missing. Check your sources..."
		exit
	fi
	echo "[OpenVPN Automation] Installed successully!"
fi

if [ ! -d $workdir ] ; then
	echo "[OpenVPN Automation] Workdir '${workdir}' is missing!"
	exit
fi

cd $workdir
chmod a-rwx,u+rwx -R $workdir

if [ "" = "$1" ] ; then
	clear
	echo "Usage: $0 <name> <country> [v4]"
	echo "Choose connection from below."
	for dir in o*
	do
		if [ -d $dir ] ; then
			echo $dir
		fi
	done
	exit
fi

isovpn=0
isServer=0
for name in *$1*
do
	# ovpn.com configurations has always ovpn in their names in path/vpn-directory.
	ovpncom=$(echo $name|grep ovpn)
	if [ "${ovpncom}" != "" ] ; then
		isovpn=1
	fi
done

vpndir=""
country="se"
connectionType=""
name=$1

if [ "$isovpn" = "1" ] ; then
	if [ "" != "$2" ] ; then
		country="$2"
	fi
	# Extra param, to use for example ipv4.
	if [ "" != "$3" ] ; then
		connectionType="-$3"
	fi
	vpndir=ovpn-${name}-${country}${connectionType}
	if [ ! -d $vpndir ] ; then
		echo "[OpenVPN Automation] VPN Directory ${vpndir} not found!"
		exit
	fi
else
	name=$1
	echo "[OpenVPN Automation] Scripts for OVPN (${name}) could not be found. Checking regular configuration."
	if [ -d ${workdir}/openvpn-${name} ] ; then
		echo "[OpenVPN Automation] Found VPN at ${workdir}: ${name}"
		vpndir=openvpn-${name}
	else
		echo "[OpenVPN Automation] Nope. Failed"
		exit
	fi
fi

config="${workdir}/${vpndir}/ovpn.conf"
preRun="${workdir}/${vpndir}/prerun.sh"
pidFile="/var/run/${vpndir}.pid"
startup=0
pingReq=0
hasRouteDev=0
killswitch=0
killswitchsetting="default mode"

if [ ! -f ${workdir}/${vpndir}/server ] ; then
	echo "[OpenVPN Automation] Client."
else
	echo "[OpenVPN Automation] Server."
	isServer=1
fi
if [ -f ${workdir}/${vpndir}/killswitch ] ; then
	killswitch=1
	killswitchsetting="requested."
fi
if [ -f ${workdir}/${vpndir}/nokillswitch ] ; then
	killswitch=0
	killswitchsetting="nokillswitch present."
fi

findtun=""
if [ ! -f $config ] ; then
	for configFile in $workdir/openvpn-$1/*conf $workdir/openvpn-$1/*ovpn
	do
		if [ -f $configFile ] ; then
			echo "[OpenVPN Automation] Using config file: ${configFile} ..."
			config=${configFile}
			findtun=$(grep ^dev ${configFile})
		fi
	done
else
	echo "[OpenVPN Automation] Fetch device from ${configFile}."
	findtun=$(grep ^dev ${config}|sed 's/^dev //')
fi

hasif=0
if [ "$findtun" = "tun" ] ; then
	useif=tun0
	hasif=1
	echo "[OpenVPN Automation] Interface is named tun, will use default name (${useif})."
fi
if [ "$findtun" = "tap" ] ; then
	useif=tap0
	hasif=1
	echo "[OpenVPN Automation] Interface is named tun, will use default name (${useif})."
fi

if [ "$hasif" = "0" ] ; then
	useif=$(echo ${findtun}|awk '{printf $2}')
	echo "FindTUN is ${findtun}"
	echo "[OpenVPN Automation] Dev is set to 'non standard' in config (${findtun}). Will adopt and use ${useif}."
fi

echo "Interface ${useif} will be used for matching interfaces."
if [ "" != "${useif}" ] ; then
	iproute=$(ip route | grep -i $useif | awk '{printf $3}')
	iprouteIf=$(ip route | grep -i $useif | awk '{printf $5}')
	iproute6=$(ip -6 route | grep -i $useif | awk '{printf $3}')
	iprouteIf6=$(ip -6 route | grep -i $useif | awk '{printf $5}')
else
	iproute=""
	iprouteIf=""
	iproute6=""
	iprouteIf6=""
fi

# DefZero is set on kali.
iprouteDefZero=$(ip route | grep "0.0.0.0/1" | awk '{printf $5}')
echo "[OpenVPN Automation] Interface to use: ${useif}. Server set to ${isServer}."

if [ "${isServer}" = "0" ] ; then
	echo "Matching ${iproute} with ${useif}."
	echo "Matching ${iprouteIf} with ${useif}."
	echo "Matching ${iproute6} with ${useif}."
	echo "Matching ${iprouteIf6} with ${useif}."
	if [ "${iprouteDefZero}" = "${useif}" ] ; then
		hasRouteDev=1
	fi
	if [ "$iproute" != "" ] ; then
		hasRouteDev=1
		echo "Found ${useif} in \$iproute."
	fi
	if [ "$iprouteIf" != "" ] ; then
		hasRouteDev=1
		echo "Found ${useif} in \$iprouteIf."
	fi
	if [ "$iproute6" != "" ] ; then
		hasRouteDev=1
		echo "Found ${useif} in \$iproute6."
	fi
	if [ "$iprouteIf6" != "" ] ; then
		hasRouteDev=1
		echo "Found ${useif} in \$iprouteIf6."
	fi
fi

if [ "${hasRouteDev}" = "1" ] ; then
	echo "[OpenVPN Automation] Eventually found a proper VPN route at a tunnel interface..."
fi

if [ -f $pidFile ] ; then
	pidId=$(cat ${pidFile})
	echo "[OpenVPN Automation] PID control (${pidFile}/${pidId}) ..."
	pidCheck=$(cat ${pidFile})
	if $(kill -CHLD $pidCheck >/dev/null 2>&1); then
		if [ "${isServer}" = "0" ] ; then
			echo "[OpenVPN Automation] Process running. Ping test request."
			pingReq=1
		else
			echo "[OpenVPN Automation] Process is running as server. No ping necessary."
		fi
	else
		echo "[OpenVPN Automation] VPN is down (pid from $pidFile: $pidCheck). Restart required."
		startup=1
	fi
else
	echo "[OpenVPN Automation] pid file (${pidFile} missing, restart!"
	startup=1
fi

echo "[OpenVPN Automation] Chosen VPN directory is ${workdir}/${vpndir}."

# 0 = Link is completely dead and gateway is reset.
# 1 = Link is alive.

if [ "${pingReq}" = "1" ] ; then
	if [ "$hasRouteDev" = "0" ] && [ "${isServer}" = "0" ] ; then
		echo "[OpenVPN Automation] Route is not tun0/tap0. Automatically requesting restart."
		startup=1
	else
		if [ "${isServer}" = "0" ] ; then
			echo "[OpenVPN Automation] Checking link, since tun/tap is available ..."
			checkLife=$(pingit $useif)
			lifeString="dead"
			if [ "${checkLife}" = "1" ] ; then
				lifeString="up"
			fi
			echo "[OpenVPN Automation] Link check response: ${lifeString} ..."
			if [ "${checkLife}" = "0" ] ; then
				echo "[OpenVPN Automation] Link seems dead. Restart required."
				startup=1
			fi
		fi
	fi
fi

if [ "${startup}" = "1" ] ; then
	# Killswitch fixer.
	echo "[OpenVPN Automation] Startup in progress ..."
	if [ "${isServer}" = "0" ] ; then
		if [ "$killswitch" = "1" ] ; then
			echo "[OpenVPN Automation] Killswitch (delete default gateway) active due to $killswitchsetting."
			ip route del default >/dev/null 2>&1 || true
		else
			echo "[OpenVPN Automation] Killswitch ignored due to $killswitchsetting."
		fi
	fi

	if [ "" != "$pkill" ] && [ -f $pkill ] ; then
		if [ "${isServer}" = "0" ] ; then
			${pkill} -9 openvpn
		else
			echo "[OpenVPN Automation] Marked as server, so not killing any process."
		fi
	else
		if [ "${killall}" != "" ] ; then
			${killall} -9 openvpn >/dev/null 2>&1
		fi
	fi

	if [ -f ${preRun} ] ; then
		echo "[OpenVPN Automation] ${preRun} script exists in directory. Run first!"
		bash ${preRun}
	fi
	echo "[OpenVPN Automation] Run: ${runthis} ${config} --writepid ${pidFile}"
	$(${runthis} ${config} --writepid ${pidFile})
fi
